@@ -61,10 +61,11 @@ GMainLoop *loop = NULL;
 
 typedef enum {
     CONNECT_STEP_FIRST,
+    CONNECT_STEP_CHECK_RUNNING,
     CONNECT_STEP_STACK_INIT,
     CONNECT_STEP_IPSEC_START,
-    CONNECT_STEP_CONFIG_ADD,
     CONNECT_STEP_WAIT_READY,
+    CONNECT_STEP_CONFIG_ADD,
     CONNECT_STEP_CONNECT,
     CONNECT_STEP_LAST
 } ConnectStep;
@@ -85,6 +86,7 @@ typedef struct {
 	gboolean libreswan;
 	gboolean interactive;
 	gboolean pending_auth;
+	gboolean managed;
 
 	GPid pid;
 	guint watch_id;
@@ -137,6 +139,7 @@ static ValidProperty valid_properties[] = {
 	{ NM_OPENSWAN_DPDTIMEOUT,                 G_TYPE_INT, 0, 86400 },
 	{ NM_OPENSWAN_IKE,                        G_TYPE_STRING, 0, 0 },
 	{ NM_OPENSWAN_ESP,                        G_TYPE_STRING, 0, 0 },
+	{ NM_OPENSWAN_VENDOR,                     G_TYPE_STRING, 0, 0 },
 	/* Ignored option for internal use */
 	{ NM_OPENSWAN_PSK_INPUT_MODES,            G_TYPE_NONE, 0, 0 },
 	{ NM_OPENSWAN_XAUTH_PASSWORD_INPUT_MODES, G_TYPE_NONE, 0, 0 },
@@ -336,7 +339,7 @@ pipe_init (Pipe *pipe, int fd, const char *detail)
 	pipe->channel = g_io_channel_unix_new (fd);
 	g_io_channel_set_encoding (pipe->channel, NULL, NULL);
 	g_io_channel_set_buffered (pipe->channel, FALSE);
-	pipe->id = g_io_add_watch (pipe->channel, G_IO_IN | G_IO_ERR, pr_cb, pipe);
+	pipe->id = g_io_add_watch (pipe->channel, G_IO_IN | G_IO_ERR | G_IO_HUP, pr_cb, pipe);
 }
 
 static void
@@ -393,8 +396,6 @@ connect_cleanup (NMOpenSwanPlugin *self)
 		g_free (priv->password);
 		priv->password = NULL;
 	}
-
-	g_clear_object (&priv->connection);
 }
 
 static void
@@ -412,12 +413,22 @@ static gboolean
 ipsec_stop (NMOpenSwanPlugin *self, GError **error)
 {
 	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (self);
-	const char *argv[4];
+	const char *argv[5];
 	guint i = 0;
 
-	delete_secrets_file (self);
+	if (!priv->connection)
+		return TRUE;
 
-	if (priv->libreswan) {
+	delete_secrets_file (self);
+	connect_cleanup (self);
+
+	if (!priv->managed) {
+		argv[i++] = priv->ipsec_path;
+		argv[i++] = "auto";
+		argv[i++] = "--delete";
+		argv[i++] = nm_connection_get_uuid (priv->connection);
+		argv[i++] = NULL;
+	} else if (priv->libreswan) {
 		argv[i++] = priv->whack_path;
 		argv[i++] = "--shutdown";
 		argv[i++] = NULL;
@@ -433,10 +444,11 @@ ipsec_stop (NMOpenSwanPlugin *self, GError **error)
 
 static void
 connect_failed (NMOpenSwanPlugin *self,
-                gboolean do_stop,
                 GError *error,
                 NMVPNConnectionStateReason reason)
 {
+	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (self);
+
 	if (error) {
 		g_warning ("Connect failed: (%s/%d) %s",
 		           g_quark_to_string (error->domain),
@@ -444,13 +456,48 @@ connect_failed (NMOpenSwanPlugin *self,
 		           error->message);
 	}
 
-	connect_cleanup (self);
-	if (do_stop)
-		ipsec_stop (self, NULL);
+	ipsec_stop (self, NULL);
+	g_clear_object (&priv->connection);
 	nm_vpn_plugin_failure (NM_VPN_PLUGIN (self), reason);
 	nm_vpn_plugin_set_state (NM_VPN_PLUGIN (self), NM_VPN_SERVICE_STATE_STOPPED);
 }
 
+static void
+check_running_cb (GPid pid, gint status, gpointer user_data)
+{
+	NMOpenSwanPlugin *self = NM_OPENSWAN_PLUGIN (user_data);
+	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (self);
+	guint ret = 1;
+	GError *error = NULL;
+
+	if (priv->watch_id == 0 || priv->pid != pid) {
+		/* Reap old child */
+		waitpid (pid, NULL, WNOHANG);
+		return;
+	}
+
+	priv->watch_id = 0;
+	priv->pid = 0;
+
+	if (WIFEXITED (status))
+		ret = WEXITSTATUS (status);
+
+	DEBUG ("Spawn: child %d exited with status %d", pid, ret);
+
+	/* Reap child */
+	waitpid (pid, NULL, WNOHANG);
+
+	if (ret)
+		priv->connect_step++;
+	else
+		priv->connect_step = CONNECT_STEP_WAIT_READY;
+
+	if (!connect_step (self, &error))
+		connect_failed (self, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
+
+	g_clear_error (&error);
+}
+
 static gboolean
 retry_cb (gpointer user_data)
 {
@@ -461,7 +508,7 @@ retry_cb (gpointer user_data)
 	priv->retry_id = 0;
 
 	if (!connect_step (self, &error))
-		connect_failed (self, TRUE, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
+		connect_failed (self, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
 	g_clear_error (&error);
 
 	return FALSE;
@@ -474,7 +521,6 @@ child_watch_cb (GPid pid, gint status, gpointer user_data)
 	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (self);
 	guint ret = 1;
 	GError *error = NULL;
-	gboolean do_stop = FALSE;
 
 	if (priv->watch_id == 0 || priv->pid != pid) {
 		/* Reap old child */
@@ -506,7 +552,6 @@ child_watch_cb (GPid pid, gint status, gpointer user_data)
 
 	if (ret == 0) {
 		/* Success; do the next connect step */
-		do_stop = TRUE;
 		priv->connect_step++;
 		priv->retries = 0;
 		if (!connect_step (self, &error))
@@ -514,7 +559,7 @@ child_watch_cb (GPid pid, gint status, gpointer user_data)
 	}
 
 	if (ret != 0)
-		connect_failed (self, do_stop, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
+		connect_failed (self, error, NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
 	g_clear_error (&error);
 }
 
@@ -655,7 +700,7 @@ nm_openswan_config_write (gint fd,
 	write_config_option (fd, " salifetime=24h\n");
 	write_config_option (fd, " ikelifetime=24h\n");
 	write_config_option (fd, " keyingtries=1\n");
-	if (libreswan)
+	if (libreswan && g_strcmp0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENSWAN_VENDOR), "Cisco") == 0)
 		write_config_option (fd, " cisco-unity=yes\n");
 	write_config_option (fd, " auto=add");
 
@@ -881,7 +926,13 @@ io_cb (GIOChannel *source, GIOCondition condition, gpointer user_data)
 	NMVPNConnectionStateReason reason = NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED;
 	const char *found;
 
-	if (condition & (G_IO_ERR | G_IO_HUP)) {
+	if (condition & G_IO_HUP) {
+		DEBUG ("PTY disconnected");
+		priv->io_id = 0;
+		return G_SOURCE_REMOVE;
+	}
+
+	if (condition & G_IO_ERR) {
 		g_warning ("PTY spawn: pipe error!");
 		goto done;
 	}
@@ -938,7 +989,7 @@ io_cb (GIOChannel *source, GIOCondition condition, gpointer user_data)
 done:
 	if (!success) {
 		priv->io_id = 0;
-		connect_failed (self, TRUE, NULL, reason);
+		connect_failed (self, NULL, reason);
 	}
 	return success ? G_SOURCE_CONTINUE : G_SOURCE_REMOVE;
 }
@@ -952,7 +1003,7 @@ pr_cb (GIOChannel *source, GIOCondition condition, gpointer user_data)
 	char *nl;
 
 	if (condition & (G_IO_ERR | G_IO_HUP)) {
-		g_warning ("PTY(%s) pipe error!", pipe->detail);
+		DEBUG ("PTY(%s) pipe error!", pipe->detail);
 		return G_SOURCE_REMOVE;
 	}
 	g_assert (condition & G_IO_IN);
@@ -1003,6 +1054,12 @@ connect_step (NMOpenSwanPlugin *self, GError **error)
 		/* fall through */
 		priv->connect_step++;
 
+	case CONNECT_STEP_CHECK_RUNNING:
+		if (!do_spawn (&priv->pid, NULL, NULL, error, priv->ipsec_path, "auto", "--status", NULL))
+			return FALSE;
+		priv->watch_id = g_child_watch_add (priv->pid, check_running_cb, self);
+		return TRUE;
+
 	case CONNECT_STEP_STACK_INIT:
 		if (priv->libreswan) {
 			const char *stackman_path;
@@ -1028,10 +1085,20 @@ connect_step (NMOpenSwanPlugin *self, GError **error)
 			                    NULL);
 		} else
 			success = do_spawn (&priv->pid, NULL, NULL, error, priv->ipsec_path, "setup", "start", NULL);
-		if (success)
+		if (success) {
+			priv->managed = TRUE;
 			priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
+		}
 		return success;
 
+	case CONNECT_STEP_WAIT_READY:
+		if (!priv->retries)
+			priv->retries = 30;
+		if (!do_spawn (&priv->pid, NULL, NULL, error, priv->ipsec_path, "auto", "--ready", NULL))
+			return FALSE;
+		priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
+		return TRUE;
+
 	case CONNECT_STEP_CONFIG_ADD:
 		if (!do_spawn (&priv->pid, &fd, NULL, error, priv->ipsec_path,
 		               "auto", "--replace", "--config", "-", uuid, NULL))
@@ -1041,14 +1108,6 @@ connect_step (NMOpenSwanPlugin *self, GError **error)
 		close (fd);
 		return TRUE;
 
-	case CONNECT_STEP_WAIT_READY:
-		if (!priv->retries)
-			priv->retries = 30;
-		if (!do_spawn (&priv->pid, NULL, NULL, error, priv->ipsec_path, "auto", "--ready", NULL))
-			return FALSE;
-		priv->watch_id = g_child_watch_add (priv->pid, child_watch_cb, self);
-		return TRUE;
-
 	case CONNECT_STEP_CONNECT:
 		if (!spawn_pty (&up_stdout, &up_stderr, &up_pty, &priv->pid, error,
 		                priv->ipsec_path, "auto", "--up", uuid, NULL))
@@ -1060,7 +1119,7 @@ connect_step (NMOpenSwanPlugin *self, GError **error)
 		priv->channel = g_io_channel_unix_new (up_pty);
 		g_io_channel_set_encoding (priv->channel, NULL, NULL);
 		g_io_channel_set_buffered (priv->channel, FALSE);
-		priv->io_id = g_io_add_watch (priv->channel, G_IO_IN | G_IO_ERR, io_cb, self);
+		priv->io_id = g_io_add_watch (priv->channel, G_IO_IN | G_IO_ERR | G_IO_HUP, io_cb, self);
 
 		if (debug) {
 			pipe_init (&priv->out, up_stdout, "OUT");
@@ -1261,8 +1320,13 @@ real_new_secrets (NMVPNPlugin *plugin,
 static gboolean
 real_disconnect (NMVPNPlugin *plugin, GError **error)
 {
-	connect_cleanup (NM_OPENSWAN_PLUGIN (plugin));
-	return ipsec_stop (NM_OPENSWAN_PLUGIN (plugin), error);
+	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (plugin);
+	gboolean ret;
+
+	ret = ipsec_stop (NM_OPENSWAN_PLUGIN (plugin), error);
+	g_clear_object (&priv->connection);
+
+	return ret;
 }
 
 static void
@@ -1273,8 +1337,11 @@ nm_openswan_plugin_init (NMOpenSwanPlugin *plugin)
 static void
 finalize (GObject *object)
 {
+	NMOpenSwanPluginPrivate *priv = NM_OPENSWAN_PLUGIN_GET_PRIVATE (object);
+
 	delete_secrets_file (NM_OPENSWAN_PLUGIN (object));
 	connect_cleanup (NM_OPENSWAN_PLUGIN (object));
+	g_clear_object (&priv->connection);
 
 	G_OBJECT_CLASS (nm_openswan_plugin_parent_class)->finalize (object);
 }
